@@ -1,7 +1,7 @@
 use strict;
 use warnings;
 
-package Text::Parser::AutoSplit 0.920;
+package Text::Parser::AutoSplit 0.925;
 
 # ABSTRACT: A role that adds the ability to auto-split a line into fields
 
@@ -9,6 +9,7 @@ use Moose::Role;
 use MooseX::CoverableModifiers;
 use String::Util qw(trim);
 use Text::Parser::Errors;
+use English;
 
 
 has _fields => (
@@ -53,6 +54,7 @@ sub field_range {
 
 sub __validate_index_range {
     my $self = shift;
+
     $self->field($_) for (@_);
     map { _pos_index( $_, $self->NF ) } __set_defaults(@_);
 }
@@ -77,6 +79,13 @@ sub _sub_field_range {
 }
 
 
+sub join_range {
+    my $self = shift;
+    my $sep  = ( @_ < 3 ) ? $LIST_SEPARATOR : pop;
+    join $sep, $self->field_range(@_);
+}
+
+
 1;
 
 __END__
@@ -91,7 +100,7 @@ Text::Parser::AutoSplit - A role that adds the ability to auto-split a line into
 
 =head1 VERSION
 
-version 0.920
+version 0.925
 
 =head1 SYNOPSIS
 
@@ -178,6 +187,16 @@ If C<$j> argument is omitted or set to C<undef>, it will be treated as C<-1> and
     $self->field_range();          # same as fields()
     $self->field_range(undef, -2); # Returns all elements omitting the last
 
+=head2 join_range
+
+This method essentially joins the return value of the C<field_range> method. It takes three arguments. The first argument is the joining string, and the other two are optional integer arguments C<$i> and C<$j> just like C<field_range> method.
+
+    $self->join_range();            # Joins all fields with $" (see perlvar)
+    $self->join_range(0, -1, '#');  # Joins with # separator
+    $self->join_range(2);           # Joins all elements starting with index 2 to the end
+                                    # with $"
+    $self->join_range(1, -2);       # Joins all elements in specified range with $"
+
 =head2 find_field
 
 This method finds an element matching a given criterion. The match is done by a subroutine reference passed as argument to this method. The subroutine will be called against each field on the line, until one matches or all elements have been checked. Each field will be available in the subroutine as C<$_>. Its behavior is the same as the C<first> function of L<List::Util>.
