@@ -1,4 +1,4 @@
-package Text::Parser::Errors 0.920;
+package Text::Parser::Errors 0.925;
 use strict;
 use warnings;
 
@@ -35,7 +35,7 @@ exception
 
 
 exception
-    FileNotReadable => 'file does not exist',
+    FileNotReadable => 'file is not readable',
     has             => [
     name => (
         is  => 'ro',
@@ -46,7 +46,7 @@ exception
 
 
 exception
-    FileNotPlainText => 'file does not exist',
+    FileNotPlainText => 'file is not a plain text file',
     has              => [
     name => (
         is  => 'ro',
@@ -90,6 +90,48 @@ exception
     extends => GenericError();
 
 
+exception ExAWK => 'a class of errors', extends => GenericError();
+
+
+exception
+    BadRuleSyntax => 'Compilation error in reading syntax',
+    has           => [
+    code => (
+        is  => 'ro',
+        isa => \&_Str,
+    ),
+    ],
+    has => [
+    msg => (
+        is  => 'ro',
+        isa => \&_Str,
+    ),
+    ],
+    has => [
+    subroutine => (
+        is  => 'ro',
+        isa => \&_Str,
+    ),
+    ],
+    extends => ExAWK();
+
+
+exception
+    IllegalRuleNoIfNoAct => 'Rule created without required components',
+    extends              => ExAWK();
+
+
+exception
+    IllegalRuleCont =>
+    'Rule cannot continue to next if action result is recorded',
+    extends => ExAWK();
+
+
+exception
+    RuleRunImproperly => 'run was called without a parser object',
+    extends           => ExAWK();
+
+
 1;
 
 __END__
@@ -104,7 +146,7 @@ Text::Parser::Errors - Exceptions for Text::Parser
 
 =head1 VERSION
 
-version 0.920
+version 0.925
 
 =head1 DESCRIPTION
 
@@ -196,6 +238,46 @@ B<line> - a string containing the content of the line with the unexpected contin
 
 =back
 
+=head2 ExAWK rule syntax related
+
+=head3 C<Text::Parser::Errors::ExAWK>
+
+All errors corresponding to the L<Text::Parser::Rule> class.
+
+=head3 C<Text::Parser::Errors::BadRuleSyntax>
+
+Generated from L<Text::Parser::Rule> class constructor or from the accessors of C<condition>, C<action>, or the method C<add_precondition>, when the rule strings specified fail to compile properly.
+
+=head4 Attributes
+
+=over 4
+
+=item *
+
+B<code> - the original rule string
+
+=item *
+
+B<msg>  - content of C<$@> after C<eval>
+
+=item *
+
+B<subroutine> - stringified form of the subroutine generated from the given C<code>.
+
+=back
+
+=head3 C<Text::Parser::Errors::IllegalRuleNoIfNoAct>
+
+Generated from constructor of the L<Text::Parser::Rule> when the rule is created with neither a C<condition> nor an C<action>
+
+=head3 C<Text::Parser::Errors::IllegalRuleCont>
+
+Generated when the rule option C<continue_to_next> of the L<Text::Parser::Rule> object is set true when C<dont_record> is false.
+
+=head3 C<Text::Parser::Errors::RuleRunImproperly>
+
+Generated from C<run> method of L<Text::Parser::Rule> is called without an object of L<Text::Parser> as argument.
+
 =head1 SEE ALSO
 
 =over 4
@@ -206,6 +288,10 @@ L<Text::Parser>
 
 =item *
 
+L<Text::Parser::Rule>
+
+=item *
+
 L<Throwable::SugarFactory>
 
 =item *
